@@ -4,6 +4,7 @@ use serde::de::{self, Deserialize, DeserializeSeed, Visitor};
 
 use crate::error::{Error, Result};
 use crate::types::Read;
+use crate::varint::{HighZeroBytesDroppedVarInt, HighZeroBytesDroppedVarIntDeserializer};
 
 impl Read for Vec<u8> {
     type Error = Error;
@@ -26,12 +27,13 @@ impl Read for Vec<u8> {
 /// BOLT deserializer
 pub struct Deserializer<'de, R: Read> {
     reader: &'de mut R,
+    saw_variable_field: bool,
 }
 
 impl<'de, R: Read> Deserializer<'de, R> {
     /// Deserialize from reader
     pub fn from_reader(reader: &'de mut R) -> Self {
-        Deserializer { reader }
+        Deserializer { reader, saw_variable_field: false }
     }
 }
 
@@ -40,16 +42,17 @@ pub fn from_vec<'a, T>(s: &'a mut Vec<u8>) -> Result<T>
 where
     T: Deserialize<'a>,
 {
-    let mut deserializer = Deserializer::from_reader(s);
-    let t = T::deserialize(&mut deserializer)?;
-    if deserializer.reader.is_empty() {
-        Ok(t)
-    } else {
-        Err(Error::TrailingBytes)
-    }
+    from_reader(s)
 }
 
-impl<'de, R: Read> Deserializer<'de, R> {
+/// Deserialize from reader
+pub fn from_reader<'a, T, R: Read>(r: &'a mut R) -> Result<T>
+    where
+        T: Deserialize<'a>,
+{
+    let mut deserializer = Deserializer::from_reader(r);
+    let t = T::deserialize(&mut deserializer)?;
+    Ok(t)
 }
 
 impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
@@ -66,6 +69,7 @@ impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
     where
         V: Visitor<'de>,
     {
+        self.ensure_did_not_see_variable_field()?;
         let result = self.read_bool()?;
         visitor.visit_bool(result)
     }
@@ -102,6 +106,7 @@ impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
     where
         V: Visitor<'de>,
     {
+        self.ensure_did_not_see_variable_field()?;
         let mut result = [0u8; 1];
         let n = self.reader.read(&mut result)?;
         if n < result.len() {
@@ -114,6 +119,7 @@ impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
     where
         V: Visitor<'de>,
     {
+        self.ensure_did_not_see_variable_field()?;
         let mut result = [0u8; 2];
         let n = self.reader.read(&mut result)?;
         if n < result.len() {
@@ -126,6 +132,7 @@ impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
     where
         V: Visitor<'de>,
     {
+        self.ensure_did_not_see_variable_field()?;
         let mut result = [0u8; 4];
         let n = self.reader.read(&mut result)?;
         if n < result.len() {
@@ -134,11 +141,17 @@ impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
         visitor.visit_u32(u32::from_be_bytes(result))
     }
 
-    fn deserialize_u64<V>(self, _visitor: V) -> Result<V::Value>
+    fn deserialize_u64<V>(self, visitor: V) -> Result<V::Value>
     where
         V: Visitor<'de>,
     {
-        unimplemented!()
+        self.ensure_did_not_see_variable_field()?;
+        let mut result = [0u8; 8];
+        let n = self.reader.read(&mut result)?;
+        if n < result.len() {
+            return Err(Error::Eof)
+        }
+        visitor.visit_u64(u64::from_be_bytes(result))
     }
 
     fn deserialize_f32<V>(self, _visitor: V) -> Result<V::Value>
@@ -194,6 +207,7 @@ impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
     where
         V: Visitor<'de>,
     {
+        self.ensure_did_not_see_variable_field()?;
         let result = self.read_bool()?;
 
         if result {
@@ -213,29 +227,36 @@ impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
     fn deserialize_unit_struct<V>(
         self,
         _name: &'static str,
-        visitor: V,
+        _visitor: V,
     ) -> Result<V::Value>
     where
         V: Visitor<'de>,
     {
-        self.deserialize_unit(visitor)
+        unimplemented!()
     }
 
     fn deserialize_newtype_struct<V>(
-        self,
-        _name: &'static str,
+        mut self,
+        name: &'static str,
         visitor: V,
     ) -> Result<V::Value>
     where
         V: Visitor<'de>,
     {
-        visitor.visit_newtype_struct(self)
+        self.ensure_did_not_see_variable_field()?;
+        if name == HighZeroBytesDroppedVarInt::NAME {
+            self.saw_variable_field = true;
+            visitor.visit_newtype_struct(HighZeroBytesDroppedVarIntDeserializer { reader: self.reader })
+        } else {
+            visitor.visit_newtype_struct(self)
+        }
     }
 
     fn deserialize_seq<V>(mut self, visitor: V) -> Result<V::Value>
     where
         V: Visitor<'de>,
     {
+        self.ensure_did_not_see_variable_field()?;
         let mut result = [0u8; 2];
         let n = self.reader.read(&mut result)?;
         if n < result.len() {
@@ -250,6 +271,7 @@ impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
     where
         V: Visitor<'de>,
     {
+        self.ensure_did_not_see_variable_field()?;
         visitor.visit_seq(SeqDeser::new(&mut self, len as u16))
     }
 
@@ -281,6 +303,7 @@ impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
     where
         V: Visitor<'de>,
     {
+        self.ensure_did_not_see_variable_field()?;
         let fields_vec = Vec::from(fields);
         visitor.visit_seq(StructDeser::new(&mut self, fields_vec))
     }
@@ -297,18 +320,18 @@ impl<'de, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<'de, R> {
         unimplemented!()
     }
 
-    fn deserialize_identifier<V>(self, visitor: V) -> Result<V::Value>
+    fn deserialize_identifier<V>(self, _visitor: V) -> Result<V::Value>
     where
         V: Visitor<'de>,
     {
-        self.deserialize_str(visitor)
+        unimplemented!()
     }
 
-    fn deserialize_ignored_any<V>(self, visitor: V) -> Result<V::Value>
+    fn deserialize_ignored_any<V>(self, _visitor: V) -> Result<V::Value>
     where
         V: Visitor<'de>,
     {
-        self.deserialize_any(visitor)
+        unimplemented!()
     }
 }
 
@@ -380,3 +403,12 @@ impl<'de, R: Read> Deserializer<'de, R> {
         Ok(result[0])
     }
 }
+
+impl<'a, 'de, R: Read> Deserializer<'de, R> {
+    fn ensure_did_not_see_variable_field(&mut self) -> Result<()> {
+        if self.saw_variable_field {
+            return Err(Error::VariableFieldNotAtEnd)
+        }
+        Ok(())
+    }
+}
