@@ -16,8 +16,10 @@ pub enum Error {
     Eof,
     /// A boolean was expected, but the byte was greater than 1
     ExpectedBoolean,
-    /// Leftover bytes after de-serializing
-    TrailingBytes,
+    /// Invalid varint
+    InvalidVarInt,
+    /// There can be at most one variable field in a struct, and it must be at end
+    VariableFieldNotAtEnd,
 }
 
 impl ser::Error for Error {
