@@ -23,7 +23,10 @@ pub mod error;
 pub mod ser;
 /// Utility types used for both directions
 pub mod types;
+/// Variable length compact integers
+pub mod varint;
 
-pub use crate::de::{from_vec, Deserializer};
+pub use crate::de::{from_reader, from_vec, Deserializer};
 pub use crate::error::{Error, Result};
 pub use crate::ser::{to_writer, to_vec, Serializer};
+pub use crate::varint::HighZeroBytesDroppedVarInt;
